#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "activate.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *file_selector_window;
extern GtkWidget *configure_window;
extern GtkWidget *about_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo camera;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];

void ActivateCapabilities()
{
  quadlet_t basic_funcs;
  int err;

  // ----------- Commander window --------------------------------------------
  
  // basic functionalities:
  if (1)
  {
    gtk_widget_set_sensitive(lookup_widget(commander_window,"power_frame"),TRUE);
    gtk_widget_set_sensitive(lookup_widget(commander_window,"power_reset"),TRUE);

    // in development:
    //gtk_widget_set_sensitive(lookup_widget(commander_window,"lock_frame"),TRUE);
    //gtk_widget_set_sensitive(lookup_widget(commander_window,"set_all_auto"),TRUE);
    //gtk_widget_set_sensitive(lookup_widget(commander_window,"set_all_man"),TRUE);
    //gtk_widget_set_sensitive(lookup_widget(commander_window,"lock_setup"),TRUE);
    //gtk_widget_set_sensitive(lookup_widget(capture_window,"capture_frame"),TRUE);
    //gtk_widget_set_sensitive(lookup_widget(capture_window,"capture_start"),TRUE);
    //gtk_widget_set_sensitive(lookup_widget(capture_window,"capture_stop"),TRUE);
    //gtk_widget_set_sensitive(lookup_widget(capture_window,"capture_single"),TRUE);
  }
  
  err=dc1394_query_basic_functionality(camera.handle,camera.id,&basic_funcs);
  if (basic_funcs & 0x1<<16)
  {
    gtk_widget_set_sensitive(lookup_widget(commander_window,"power_on"),TRUE);
    gtk_widget_set_sensitive(lookup_widget(commander_window,"power_off"),TRUE);
  }

  // trigger capabilities:
  gtk_widget_set_sensitive(lookup_widget(commander_window,"trigger_frame"),TRUE);
  UpdateTriggerModeMenu();

  // Update dependent menus:
  ActivateFormatMenu();
  UpdateFpsMenu();
  // a function to create the memory channel menu should be added here.

  // update ranges (min, max, current, capabilities)
  ActivateRangeCapabilities(porthole_window, FEATURE_FOCUS);
  ActivateRangeCapabilities(porthole_window, FEATURE_PAN);
  ActivateRangeCapabilities(porthole_window, FEATURE_TILT);
  ActivateRangeCapabilities(porthole_window, FEATURE_ZOOM);

  ActivateRangeCapabilities(aperture_window, FEATURE_EXPOSURE);
  ActivateRangeCapabilities(aperture_window, FEATURE_IRIS);
  ActivateRangeCapabilities(aperture_window, FEATURE_SHUTTER);
  ActivateRangeCapabilities(aperture_window, FEATURE_GAIN);
  ActivateRangeCapabilities(aperture_window, FEATURE_OPTICAL_FILTER);

  ActivateRangeCapabilities(color_window, FEATURE_BRIGHTNESS);
  ActivateRangeCapabilities(color_window, FEATURE_GAMMA);
  ActivateRangeCapabilities(color_window, FEATURE_SATURATION);
  ActivateRangeCapabilities(color_window, FEATURE_HUE);
  ActivateRangeCapabilities(color_window, FEATURE_WHITE_BALANCE);
  ActivateRangeCapabilities(color_window, FEATURE_SHARPNESS);

  ActivateRangeCapabilities(capture_window, FEATURE_CAPTURE_SIZE);
  ActivateRangeCapabilities(capture_window, FEATURE_CAPTURE_QUALITY);

  ActivateRangeCapabilities(temperature_window, FEATURE_TEMPERATURE);

}


void ActivateFormatMenu()
{
  int err;
  quadlet_t modes, formats;

  // formats capabilities:
  err= dc1394_query_supported_formats(camera.handle, camera.id, &formats);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_0"),formats & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_1"),formats & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_2"),formats & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_6"),formats & (0x1<<25) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"format_7"),formats & (0x1<<24) );

  // corresponding modes:
  err= dc1394_query_supported_modes(camera.handle, camera.id, 0, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f0_mode5"),modes & (0x1<<26) );
  
  err= dc1394_query_supported_modes(camera.handle, camera.id, 1, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f1_mode5"),modes & (0x1<<26) );
  
  err= dc1394_query_supported_modes(camera.handle, camera.id, 2, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f2_mode5"),modes & (0x1<<26) );

  err= dc1394_query_supported_modes(camera.handle, camera.id, 6, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f6_mode0"),modes & (0x1<<31) );

  err= dc1394_query_supported_modes(camera.handle, camera.id, 7, &modes);
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode0"),modes & (0x1<<31) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode1"),modes & (0x1<<30) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode2"),modes & (0x1<<29) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode3"),modes & (0x1<<28) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode4"),modes & (0x1<<27) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode5"),modes & (0x1<<26) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode6"),modes & (0x1<<25) );
  gtk_widget_set_sensitive(lookup_widget(commander_window,"f7_mode7"),modes & (0x1<<24) );

}

void ActivateRangeCapabilities(GtkWidget* current_window, int feature)
{
  GtkAdjustment *adjustment, *adjustment2;
  int minvalue, maxvalue;
  dc1394bool_t test_boolean;
  int err;

  err=dc1394_is_feature_present(camera.handle,camera.id,feature,&test_boolean);
  if (test_boolean)
    {
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_frame_list[feature]),TRUE);
      // one push auto:
      err=dc1394_has_one_push_auto(camera.handle,camera.id,feature,&test_boolean);
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_op_list[feature]),test_boolean);
      // on/off switching:
      err=dc1394_can_turn_on_off(camera.handle,camera.id,feature,&test_boolean);
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_power_list[feature]),test_boolean);
      // manual mode:
      err=dc1394_has_manual_mode(camera.handle,camera.id,feature,&test_boolean);
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_man_list[feature]),test_boolean);
      // auto mode:
      err=dc1394_has_auto_mode(camera.handle,camera.id,feature,&test_boolean);
      gtk_widget_set_sensitive(lookup_widget(current_window, feature_auto_list[feature]),test_boolean);
      // min/max values:
      err=dc1394_get_min_value(camera.handle,camera.id,feature,&minvalue);
      err=dc1394_get_max_value(camera.handle,camera.id,feature,&maxvalue);

      switch(feature)
	{
	  case FEATURE_WHITE_BALANCE:
	    adjustment=(GtkAdjustment*)gtk_adjustment_new(minvalue,minvalue,maxvalue,1,10,0);
	    // defaulted to minvalue, will be updated by update function.
	    adjustment2=(GtkAdjustment*)gtk_adjustment_new(minvalue,minvalue,maxvalue,1,10,0);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "whitebal_BU_scale"),adjustment);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "whitebal_RV_scale"),adjustment2);break;
	  case FEATURE_TEMPERATURE:
	    printf("hello world!\n");
	    adjustment=(GtkAdjustment*)gtk_adjustment_new(minvalue,minvalue,maxvalue,1,10,0);
	    // defaulted to minvalue, will be updated by update function.
	    adjustment2=(GtkAdjustment*)gtk_adjustment_new(minvalue,minvalue,maxvalue,1,10,0);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "temp_goal_scale"),adjustment);
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, "temp_current_scale"),adjustment2);
	    printf("hello world!\n");break;
	  default:
	    adjustment=(GtkAdjustment*)gtk_adjustment_new(minvalue,minvalue,maxvalue,1,10,0);
	    // defaulted to minvalue, will be updated by update function.
	    gtk_range_set_adjustment((GtkRange*)lookup_widget(current_window, feature_scale_list[feature]),adjustment);
	}
    }

  UpdateRange(current_window,feature);
}
