#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "callback_proc.h"
#include <libdc1394/dc1394_control.h>
#include <libraw1394/raw1394.h>
#include "video1394.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *file_selector_window;
extern GtkWidget *configure_window;
extern GtkWidget *about_window;
extern GtkWidget *capture_window;
extern GtkWidget *temperature_window;
extern dc1394_camerainfo camera;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];

void OpRangeProcedure(GtkButton* widget, int feature)
{
  int err;
  dc1394bool_t value=TRUE;

  err=dc1394_start_one_push_operation(camera.handle, camera.id, feature);
  gtk_widget_set_sensitive(GTK_WIDGET (widget), FALSE);
  while (value)
    {
      err=dc1394_is_one_push_in_operation(camera.handle, camera.id, feature, &value);
      UpdateRangeValue(GTK_WIDGET (widget),feature);
    }
  gtk_widget_set_sensitive(GTK_WIDGET (widget),1);
}

void ManRangeProcedure(GtkToggleButton *widget, int feature)
{
  int err;
  dc1394bool_t has_op;

  // sensitive OP function if available
  err=dc1394_has_one_push_auto(camera.handle, camera.id, feature, &has_op);
  gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), feature_op_list[feature])),has_op);

  switch (feature)
    {
      case FEATURE_WHITE_BALANCE:
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), "whitebal_RV_scale")),TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), "whitebal_BU_scale")),TRUE);break;
      case FEATURE_TEMPERATURE:
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), "temp_goal_scale")),TRUE);break;
	// only the goal range is active, the other one is just an indicator
      default:
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), feature_scale_list[feature])),TRUE);
    }
  // Also, man mode should disable the update of the range widgets
  err=dc1394_auto_on_off(camera.handle, camera.id, feature, 0);

}

void AutoRangeProcedure(GtkToggleButton *widget, int feature)
{
  int err;

  // unsensitive OP and range function:
  gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), feature_op_list[feature])),FALSE);
  switch(feature)
    {
      case FEATURE_WHITE_BALANCE:
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), "whitebal_RV_scale")),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), "whitebal_BU_scale")),FALSE);break;
      case FEATURE_TEMPERATURE:
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), "temp_goal_scale")),FALSE);break;
      default:
	gtk_widget_set_sensitive(GTK_WIDGET (lookup_widget(GTK_WIDGET (widget), feature_scale_list[feature])),FALSE);
    }
  // Also, auto mode should update regularly the values of the range widgets
  err=dc1394_auto_on_off(camera.handle, camera.id, feature, 1);

}

void PowerRangeProcedure(GtkToggleButton* widget, int feature)
{
  int err;
  if (GTK_TOGGLE_BUTTON (widget)->active)
    err=dc1394_feature_on_off(camera.handle, camera.id, feature, 1);
  else
    err=dc1394_feature_on_off(camera.handle, camera.id, feature, 0);
}
